﻿using Bedelia.Dtos;
using Bedelia.EF;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Bedelia.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class UsersController : ControllerBase
    {
        private readonly BedeliaContext _context;

        public UsersController(BedeliaContext context)
        {
            _context = context;
        }

        [HttpGet]
        public async Task<IActionResult> Get()
        {
            var users = await _context.Users.ToListAsync();
            return Ok(users);
        }

        [HttpGet("{identityCard}")]
        public async Task<IActionResult> Get(string identityCard)
        {
            var user = await _context.Users.FirstOrDefaultAsync(x => x.IdentityCard == identityCard);

            if (user == null)
            {
                return BadRequest(string.Format("el usuario con cedula {0} no existe en bedelia", identityCard));
            }

            return Ok(user);
        }

        [HttpPost]
        public async Task<IActionResult> Post([FromBody] UserCreateDto userDto)
        {
            var user = new User()
            {
                IdentityCard = userDto.IdentityCard,
                Name = userDto.Name,
                Password = userDto.Password
            };

            _context.Users.Add(user);
            await _context.SaveChangesAsync();

            return Ok(user);
        }

        [HttpPost("loadData")]
        public async Task<IActionResult> Post([FromBody] List<UserCreateDto> userDtos)
        {
            var userIdentityCards = userDtos.Select(x => x.IdentityCard);

            var dbUsers = await _context.Users
                .Where(x => userIdentityCards.Contains(x.IdentityCard))
                .ToListAsync();

            var users = new List<User>();
            foreach (var userDto in userDtos)
            {
                var dbUser = dbUsers.FirstOrDefault(x => x.IdentityCard == userDto.IdentityCard);
                if (dbUser == null)
                {
                    users.Add(new User()
                    {
                        IdentityCard = userDto.IdentityCard,
                        Name = userDto.Name,
                        Password = userDto.Password
                    });
                }
            }

            _context.Users.AddRange(users);
            await _context.SaveChangesAsync();

            return Ok(users);
        }

        [HttpPut("{id}")]
        public async Task<IActionResult> Put(int id, [FromBody] User user)
        {
            var dbUser = await _context.Users.FirstOrDefaultAsync(x => x.Id == id);

            if (dbUser == null)
            {
                return BadRequest(string.Format("el usuario con id {0} no existe en bedelia", id));
            }

            dbUser.IdentityCard = user.IdentityCard;
            dbUser.Name = user.Name;
            dbUser.Password = user.Password;

            await _context.SaveChangesAsync();

            return Ok(dbUser);
        }

        [HttpDelete("{id}")]
        public async Task<IActionResult> Delete(int id)
        {
            var user = await _context.Users.FirstOrDefaultAsync(x => x.Id == id);

            _context.Users.Remove(user);
            await _context.SaveChangesAsync();

            return Ok();
        }

        [HttpGet("userNotes")]
        public async Task<IActionResult> UserNotes()
        {
            var result = await (from user in _context.Users
                                join userCourse in _context.UserCourses
                                    on user.Id equals userCourse.UserId
                                join course in _context.Courses
                                    on userCourse.CourseId equals course.Id
                                select new
                                {
                                    User = user.IdentityCard,
                                    Course = course.Name,
                                    Grade = userCourse.Grade,
                                    GradeDate = userCourse.GradeDate
                                }).ToListAsync();

            return Ok(result);
        }

    }
}
