﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;
using Tsi1.DataLayer;
using Tsi1.DataLayer.Entities;

namespace Tsi1.BusinessLayer.DataLoad
{
    public class DataLoad : IDataLoad
    {
        private readonly Tsi1Context _context;
        private readonly ITenantService _tenantService;
        private readonly IUserService _userService;
        private readonly ICourseService _courseService;
        private readonly ISectionService _sectionService;
        private readonly ISectionItemService _sectionItemService;

        public DataLoad(Tsi1Context context,
            ITenantService tenantService,
            IUserService userService,
            ICourseService courseService,
            ISectionService sectionService,
            ISectionItemService sectionItemService)
        {
            _context = context;
            _tenantService = tenantService;
            _userService = userService;
            _courseService = courseService;
            _sectionService = sectionService;
            _sectionItemService = sectionItemService;
        }

        public async Task LoadDataAsync()
        {
            // TENANTS
            var tenants = new List<Tenant>
            {
                new Tenant
                {
                    Name = TenantAdmin.Name,
                },
                new Tenant
                {
                    Name = "fing",
                },
            };
            _context.Tenants.AddRange(tenants);

            await _context.SaveChangesAsync();

            // USER TYPES
            var udelarAdminUserType = new UserType
            {
                Name = UserTypes.UdelarAdmin,
            };
            _context.Add(udelarAdminUserType);

            var facultyAdminUserType = new UserType
            {
                Name = UserTypes.FacultyAdmin,
            };
            _context.Add(facultyAdminUserType);

            var professorUserType = new UserType
            {
                Name = UserTypes.Professor,
            };
            _context.Add(professorUserType);

            var studentUserType = new UserType
            {
                Name = UserTypes.Student,
            };
            _context.Add(studentUserType);

            await _context.SaveChangesAsync();

            // USERS
            var users = new List<User>
            {
                new User
                {
                    UserTypeId = udelarAdminUserType.Id,
                    Username = "admin",
                    Password = "admin",
                    FirstName = "admin",
                    LastName = "admin",
                    Email = "admin@mail.com",
                    TenantId = 1,
                },

                new User
                {
                    UserTypeId = studentUserType.Id,
                    Username = "enzo",
                    Password = "enzo",
                    FirstName = "Enzo",
                    LastName = "Santangelo",
                    Email = "enzo@mail.com",
                    Student = new Student
                    {
                        TenantId = 2,
                        IdentityCard = "13242344-5",
                        Age = 25
                    },
                    TenantId = 2,
                },

                new User
                {
                    UserTypeId = studentUserType.Id,
                    Username = "mathias",
                    Password = "mathias",
                    FirstName = "Mathias",
                    LastName = "Martinez",
                    Email = "mathias@mail.com",
                    Student = new Student
                    {
                        IdentityCard = "3782346-5",
                        TenantId = 2,
                        Age = 26
                    },
                    TenantId = 2,
                },

                new User
                {
                    UserTypeId = professorUserType.Id,
                    Username = "prof1",
                    Password = "prof1",
                    FirstName = "Juan",
                    LastName = "Perez",
                    Email = "jp@mail.com",
                    Professor = new Professor
                    {
                        TenantId = 2,
                        IdentityCard = "98754342-5",
                    },
                    TenantId = 2,
                },

                new User
                {
                    UserTypeId = facultyAdminUserType.Id,
                    Username = "fing",
                    Password = "fing",
                    FirstName = "fing",
                    LastName = "fing",
                    Email = "fing@mail.com",
                    TenantId = 2,
                },
            };

            _context.Users.AddRange(users);

            try
            {
                await _context.SaveChangesAsync();
            }
            catch (Exception ex)
            {

                Console.WriteLine(ex.Message);
            }

            // SECTION ITEM TYPES
            var sectionItemTypeFile = new SectionItemType
            {
                Name = SectionItemTypes.File,
            };
            _context.SectionItemTypes.Add(sectionItemTypeFile);

            var sectionItemTypeForum = new SectionItemType
            {
                Name = SectionItemTypes.Forum,
            };
            _context.SectionItemTypes.Add(sectionItemTypeForum);

            var sectionItemTypeSurvey = new SectionItemType
            {
                Name = SectionItemTypes.Survey,
            };
            _context.SectionItemTypes.Add(sectionItemTypeSurvey);

            await _context.SaveChangesAsync();

            // COURSES  
            var courses = new List<Course>
            {
                new Course
                {
                    Id = 1,
                    Name = "Calculo 1",
                    TenantId = 2
                },

                new Course
                {
                    Id = 2,
                    Name = "GAL 1",
                    TenantId = 2
                },

                new Course
                {
                    Id = 3,
                    Name = "Fisica 1",
                    TenantId = 2
                }
            };

            _context.Courses.AddRange(courses);

            await _context.SaveChangesAsync();


            // SECTIONS
            var section1 = new Section
            {
                CourseId = 1,
                Name = "General",
                Order = 1,
                SectionItems = new List<SectionItem>
                {
                    new SectionItem
                    {
                        Order = 1,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "Novedades"
                        }
                    },
                    new SectionItem
                    {
                        Order = 2,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "General"
                        }
                    },
        }
            };
            _context.Sections.Add(section1);

            var section2 = new Section
            {
                CourseId = 2,
                Name = "General",
                Order = 1,
                SectionItems = new List<SectionItem>
                {
                    new SectionItem
                    {

                        Order = 1,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "Novedades"
                        }
                    }
                }
            };
            _context.Sections.Add(section2);

            var section3 = new Section
            {
                CourseId = 1,
                Name = "Tema 1",
                Order = 2,
                SectionItems = new List<SectionItem>
                {
                    new SectionItem
                    {
                        Order = 1,
                        SectionItemTypeId = sectionItemTypeForum.Id,
                        Forum = new Forum
                        {
                            Name = "Tema 1"
                        }
                    }
                }
            };
            _context.Sections.Add(section3);

            await _context.SaveChangesAsync();

            // ANSWER OPTIONS
            var answerOptions = new List<AnswerOption>
            {
                new AnswerOption
                {
                    Name = "Excelente"
                },
                new AnswerOption
                {
                    Name = "Conforme"
                },
                new AnswerOption
                {
                    Name = "Neutro"
                },
                new AnswerOption
                {
                    Name = "No Conforme"
                },
                new AnswerOption
                {
                    Name = "Mal"
                },
            };

            _context.AnswerOptions.AddRange(answerOptions);
            await _context.SaveChangesAsync();

        }
    }
}